using System;
using System.Collections;
using System.Text;

namespace at.jku.ssw.Analyse {
	/// <summary>
	/// Diese Klasse speichert die Informationen fr die Berechnung der Kennzahlen
	/// nach Halstead
	/// </summary>
	public class HalsteadInfo {
		private Hashtable htOperators; // Vorkommnisse der verschiedenen Operatoren
		private Hashtable htOperands; // Vorkommnisse der verschiedenen Oberanden

		// von Halstead definierte Kennzahlen
		public int n1, N1, n2, N2;
		public int n, N;
		public float V, D, E, L, T;

		public HalsteadInfo() {
			htOperators = new Hashtable();
			htOperands = new Hashtable();
		}

		/// <summary>
		/// Fgt einen Operator in die Hashtable htOperators hinzu, bzw. erhht
		/// seinen Zhler, falls er schon vorgekommen ist.
		/// </summary>
		public void addOperator(String s) {
			if (htOperators.ContainsKey(s))
				htOperators[s] = (int) htOperators[s] + 1;
			else htOperators.Add(s, 1);
		}

		/// <summary>
		/// Fgt einen Operand in die Hashtable htOperands hinzu, bzw. erhht
		/// seinen Zhler, falls er schon vorgekommen ist.
		/// </summary>
		public void addOperand(String s) {
			if (htOperands.ContainsKey(s))
				htOperands[s] = (int)htOperands[s] + 1;
			else htOperands.Add(s, 1);
		}

		/// <summary>
		/// Falls der Parser einen Right-Shift erkennt, mssen zwei '>' Operatoren
		/// aus der Hashtable htOperators entfernt werden, und ein Right-Shift '>>'
		/// hinzugefgt werde. Dies bernimmt diese Methode.
		/// </summary>
		public void replaceRShift() {
			int i = (int)htOperators[">"] - 2;
			if (i==0) htOperators.Remove(">");
			else htOperators[">"] = i;

			addOperator(">>");
		}

		/// <summary>
		/// Der + Operator ist notwendig, um Halstead-Informationen eines Bereiches zu
		/// dem darberliegenden hinzuaddieren zu knnen.
		/// </summary>
		public static HalsteadInfo operator+(HalsteadInfo h1, HalsteadInfo h2) {
			HalsteadInfo ret = new HalsteadInfo();

			ret.htOperators = (Hashtable)h1.htOperators.Clone();
			foreach(DictionaryEntry o in h2.htOperators) {
				if (ret.htOperators.ContainsKey(o.Key))
					ret.htOperators[o.Key] = (int)ret.htOperators[o.Key] + (int)o.Value;
				else ret.htOperators.Add(o.Key, o.Value);
			}
			ret.htOperands = (Hashtable)h1.htOperands.Clone();
			foreach(DictionaryEntry o in h2.htOperands) {
				if (ret.htOperands.ContainsKey(o.Key))
					ret.htOperands[o.Key] = (int)ret.htOperands[o.Key] + (int)o.Value;
				else ret.htOperands.Add(o.Key, o.Value);
			}

			return ret;
		}

		/// <summary>
		/// Diese Methode wird am Ende eines Analyse-Durchlaufes aufgerufen, und
		/// berechnet die Kennzahlen nach Halstead aufgrund der gesammelten Daten.
		/// </summary>
		public void compute() {
			n1 = 0;
			N1 = 0;
			foreach(DictionaryEntry o in htOperators) {
				n1++;
				N1 += (int)o.Value;
			}
			n2 = 0;
			N2 = 0;
			foreach(DictionaryEntry o in htOperands) {
				n2++;
				N2 += (int)o.Value;
			}

			n = n1+n2;							//Vokabular
			N = N1+N2;							//Lnge
			if (n != 0)
				V = (float) N * (float) Math.Log((double)n, 2.0);	//Volumen
			else V = 0.0f;
			D = ((float)n1/2.0f) * ((float)N2/(float)n2);			//Schwierigkeit
			E = D * V;							//Aufwand
			if (n1 != 0 && N2 != 0)
				L = (2.0f/(float)n1) * ((float)n2/(float)N2);		//Level
			else L = 0.0f;
			T = E/18.0f;						//Programmierzeit
		}

		public String ToString(StringBuilder tab) {
			StringBuilder sb = new StringBuilder();
			sb.Append(tab).Append("Halstead's Infos\n");
			tab.Append(" ");
			sb.Append(tab).AppendFormat("n1             = {0};\tn2             = {1}\n", n1, n2);
			sb.Append(tab).AppendFormat("N1             = {0};\tN2             = {1}\n", N1, N2);
			sb.Append(tab).AppendFormat("n (Vokabulary) = {0};\tN (Length)     = {1}\n", n, N);
			sb.Append(tab).AppendFormat("V (Volume)     = {0};\tD (Difficulty) = {1}\n", V, D);
			sb.Append(tab).AppendFormat("E (Effort)     = {0};\tL (Level)      = {1}\n", E, L);
			sb.Append(tab).AppendFormat("T (Time)       = {0}\n", T);
			return sb.ToString();
		}
	}
}
